package umdes;

import java.util.Vector;

public class Cluster implements Cloneable{
	
	private UmdesAutomaton automata;
	private Vector failure;
	private Vector policy;
	
	protected boolean isInitial;
	protected State  startState; //the start state of the cluster
	protected Vector state_vec;  //the states in the cluster
	protected Vector<Transition> trans_out;  //all transitions out of the cluster
	//protected Vector trans_in;
	protected Vector state_out;  //possible destination states
	
	protected Vector X_Nfk;      //only meaningful for Cp, is part of the set of states that  
	protected Vector X_fk;   
	
	protected boolean Cp_cycle;
	protected boolean Cp_fcycle;
	protected boolean Nfk;
	
	protected int id;
	
	public Cluster(State s, UmdesAutomaton automata_in, Vector policy_in, Vector failure_in, boolean isPsuedo){
		isInitial = false;
		failure = failure_in;
		Vector<Transition> trans = new Vector<Transition>();
		state_vec = new Vector();
		trans_out = new Vector();
		state_out = new Vector();
		Nfk = true;
		policy = policy_in;
		startState = s;
		automata = automata_in;
		state_vec.add(s);
		
		if(isPsuedo){
			X_fk  = new Vector();
			X_Nfk = new Vector();
		}

		
		for(int i = 0; i < state_vec.size(); i++){
			
			try{
				trans = automata.getEvents().getTransitions((State) state_vec.get(i));
			} catch(Exception e) { return;}
			
			for(int j = 0; j< trans.size(); j++){
				if(failure.contains(((Transition) trans.get(j)).label.getEvent())&&!isPsuedo) continue;
				if(policy.contains((Transition) trans.get(j))||automata.isUnobserv(((Transition) trans.get(j)).label.getEvent())){
					if(!state_vec.contains(((Transition) trans.get(j)).to))  state_vec.add(((Transition) trans.get(j)).to);
				}
				else{
					trans_out.add(trans.get(j));
					if(!state_out.contains(((Transition) trans.get(j)).to)) state_out.add(((Transition) trans.get(j)).to);
				}
			}			
		}
		//find the X_Nfk and X_fk for this state
		if(isPsuedo){
			getX();	
			findCpCycles();
		}
		
		

	}
	
	private void findCpCycles(){
		
		Vector statesCycle = new Vector();
		Vector trans = new Vector();
		
		boolean seenFk = false;  //if there is no failure event in the cluster there is no need to look for Cp_fcycles
		statesCycle.add(startState);
		
		for(int i = 0; i < statesCycle.size(); i++){
			try{
				trans = automata.getEvents().getTransitions((State) state_vec.get(i));
			} catch(Exception e) { return;}			
			for(int j = 0; j< trans.size(); j++){
				if(policy.contains((Transition) trans.get(j))||automata.isUnobserv(((Transition) trans.get(j)).label.getEvent())){
					if(statesCycle.contains(((Transition) trans.get(j)).to)){
						Cp_cycle = true;
					}
					else statesCycle.add(((Transition) trans.get(j)).to);
					if(failure.contains(((Transition) trans.get(j)).label.getEvent())) seenFk = true; 
				}				
			}
		}
		
		if(seenFk&&Cp_cycle){
			findCpfCycle();
		}
			
	}
	
	private void findCpfCycle(){//State head, State cur, boolean seenFk){

		if(X_fk==null || X_fk.size()==0){ Cp_fcycle =  false; return;}  //this should never happen
		
		Vector stateHelp = new Vector();
		Vector trans = new Vector();
		//for there to be a C_fk cycle it must occur only in the X_fk states
		for(int i = 0; i < X_fk.size(); i++){
			State seedState = (State) X_fk.get(i);
			stateHelp.clear();
			int counter = 0;
			do {
				try{
					if(stateHelp.size()!=0){ //only increment counter when it isn't the first one
						trans = automata.getEvents().getTransitions((State) stateHelp.get(counter));
						counter++;
					}
					else trans = automata.getEvents().getTransitions(seedState);
				} catch(Exception e) {}		
				if(trans==null ||trans.size()==0) continue;
				for(int j = 0; j < trans.size(); j++){
					if(!stateHelp.contains(((Transition) trans.get(j)).to)) stateHelp.add(((Transition) trans.get(j)).to);
				}
			} while( counter < stateHelp.size());
			
			if(stateHelp.contains(seedState)) {Cp_fcycle = true; return;}
		}
		Cp_fcycle = false;

	}
	
	private void getX(){
		
		Vector trans = new Vector();
		X_Nfk.add(startState);
		
		for(int i = 0; i < X_Nfk.size(); i++){
			try{
				trans = automata.getEvents().getTransitions((State) X_Nfk.get(i));
			} catch(Exception e) {}	
			if(trans==null||trans.size()==0) continue;
			for(int j = 0; j < trans.size(); j++){
				//completes X_Nfk
				//these if statements may not be right
				if((policy.contains((Transition) trans.get(j))||automata.isUnobserv(((Transition) trans.get(j)).label.getEvent()))&&!failure.contains(((Transition) trans.get(j)).label.getEvent())){
					if(!X_Nfk.contains(((Transition) trans.get(j)).to)){			
						X_Nfk.add(((Transition) trans.get(j)).to);
					}
				}			
				//will put only the states directly after a fault transtion in X_fk
				if(failure.contains(((Transition) trans.get(j)).label.getEvent())){
					if(!X_fk.contains(((Transition) trans.get(j)).to)){			
						X_fk.add(((Transition) trans.get(j)).to);						
					}
				}				
			}
		}
		
		//propagate label for X_fk
		for(int i = 0; i < X_fk.size(); i++){
			try{
				trans = automata.getEvents().getTransitions((State) X_fk.get(i));
			} catch(Exception e) {}	
			if(trans==null||trans.size()==0) continue;	
			for(int j = 0; j < trans.size(); j++){
				if((policy.contains((Transition) trans.get(j))||automata.isUnobserv(((Transition) trans.get(j)).label.getEvent()))||failure.contains(((Transition) trans.get(j)).label.getEvent())){
					if(!X_fk.contains(((Transition) trans.get(j)).to))  X_fk.add(((Transition) trans.get(j)).to);					
				}
			}
		}
		
		
	}
	
	
	public boolean equals(Object o){
		
		if(o==null)return false;
		Cluster c_tmp;
		if (o instanceof Cluster) {
			c_tmp = (Cluster) o;
		}
		else return false;	
		if(!this.state_vec.equals(c_tmp.state_vec)) return false;
		if(!this.startState.equals(c_tmp.startState)) return false;
		if(this.Nfk!=c_tmp.Nfk) return false;
		
		return true;
	
	}

	public int hashCode(){
		int hash = 0;
		hash = trans_out.hashCode()+ startState.hashCode();
		return hash;
	}
	
	protected Object clone() throws CloneNotSupportedException {

	    Cluster clone = (Cluster)super.clone();

	    clone.state_vec = (Vector)this.state_vec.clone();
	    clone.startState = (State)this.startState.clone();
	    
	    return clone;

	 }
	
	public String toString() {
		return state_vec.toString() + " -> " + state_out.toString();
	}


}
