package umdes;

import java.util.Vector;

public class ClusterAutomata {
	
	
	protected Cluster initial;    //the initial cluster
	protected Cluster curCluster;
	protected Vector<Cluster> clusters;    //vector of all clusters
	protected Vector trans;
	protected Vector policy;      //if transition is in here it is unobservable
	protected UmdesAutomaton machine;         //the underlying automata
	protected Vector failure;     //a vector of RegExpr's that are failure events
	public int auto_id;		 	  //psuedo will be 0 while the others will increment up by one
	static private int internal_auto_num = 1;
	
	public boolean isPsuedo;
	
	/**
	 * 
	 * @param machine_in  :The automata that is cluster automata represents
     * @param trans_policy  :A Vector of all Transition's in automata
	 * @param trans_policy  :A Vector of Transition's that are Unobservable 
	 * @param failure_events  :A Vector of RegExpr's that are part of the failure partion
	 */
	public ClusterAutomata(UmdesAutomaton machine_in, Vector all_trans, Vector trans_policy, Vector failure_events){
		
		auto_id = internal_auto_num;
		internal_auto_num++;
		failure = failure_events;
		machine = machine_in;
		trans = all_trans;
		policy = trans_policy;
		isPsuedo = false;
		clusters = new Vector<Cluster>();
		constructAutomata(machine.getInitial());
		
		curCluster = initial;
	}
	
	public ClusterAutomata(Vector clust_auto){
		
		if(clust_auto==null||clust_auto.size()==0) return;
		auto_id = 0;
		//finds the policy of the C_p
		trans = ((ClusterAutomata) clust_auto.get(0)).trans;
		policy = (Vector) ((ClusterAutomata) clust_auto.get(0)).policy.clone();
		clusters = new Vector();
			
		for(int i = 1; i < clust_auto.size(); i++){	
			for(int k = 0; k < policy.size(); k++){
				if(policy.get(k)==null) continue;
				if(!((ClusterAutomata) clust_auto.get(i)).policy.contains(policy.get(k))) {policy.remove(k); k--;}
			}
		}
		
		isPsuedo = true;
		
		//all Cluster automata will have the same underlying automata
		failure = ((ClusterAutomata) clust_auto.get(0)).failure;
		machine = ((ClusterAutomata) clust_auto.get(0)).machine;
		constructAutomata(machine.getInitial());	
		curCluster = initial;

	}
	
	/**
	 * only use as a temporary ClusterAutomata (i.e. to reset_id)
	 */
	public ClusterAutomata(){}
	
	protected void reset_id(){
		internal_auto_num = 1;
	}
	
	
	//will return a vector of clusters that are reached from this cluster through the RegExpr e
	public Vector<Cluster> R(Transition t, Cluster cur){
		
		//changed all curCluster -> cur (input)
		
		RegExpr r = t.label.getEvent();
		Vector<Cluster> v = new Vector<Cluster>();
		Cluster c = cur; 
		
		if(policy.contains(t)&&cur.id!=0){
			v.add(c);
			return v;
		}
		boolean containsTrans = false;
		for(int i = 0; i < cur.trans_out.size(); i++){
			if(((Transition) cur.trans_out.get(i)).label.getEvent().equals(r)){ containsTrans = true; break;}
		}
		if(!containsTrans) return null;
		
		for(int i = 0; i< cur.trans_out.size(); i++){
		
			if(((Transition) cur.trans_out.get(i)).label.getEvent().equals(r)){

				if(isPsuedo){

					if(cur.X_fk.contains( ((Transition) cur.trans_out.get(i)).from)||!cur.Nfk){
						c = findCluster_Nfk(((Transition) cur.trans_out.get(i)).to, false);
						if(c==null) continue;
					} 
					if(cur.X_Nfk.contains(((Transition) cur.trans_out.get(i)).from)&&cur.Nfk){
						c = findCluster_Nfk(((Transition) cur.trans_out.get(i)).to, true);
						if(c==null) continue;					
					}				
				} 
				else{
					c = findCluster(((Transition) cur.trans_out.get(i)).to);
					if(c==null) continue;  //this shouldn't happen	
				}

				v.add(c);
				
			} 		
		}
		return v;
	}
	
	private Cluster findCluster(State init){
		for(int i = 0; i < clusters.size(); i++){
			if(((Cluster) clusters.get(i)).startState.equals(init)) return (Cluster) clusters.get(i);
		}
		return null;
	}
	
	private Cluster findCluster_Nfk(State init, boolean Nfk){
		for(int i = 0; i < clusters.size(); i++){
			if(((Cluster) clusters.get(i)).startState.equals(init)&&((Cluster) clusters.get(i)).Nfk==Nfk) return (Cluster) clusters.get(i);
		}
		return null;	
		
	}
	
	private void constructAutomata(State initial){
		
		State s;
		Cluster tmp = new Cluster(initial, machine, policy, failure, isPsuedo);
		tmp.isInitial = true;
		tmp.Nfk = true;
		tmp.id = auto_id;
		this.initial = tmp;	
		clusters.add(tmp);
		
		for(int i = 0; i < clusters.size(); i++){
			for(int j = 0; j < ((Cluster) clusters.get(i)).state_out.size(); j++){
				s = (State) ((Cluster) clusters.get(i)).state_out.get(j);
				if(!alreadyCluster(s)){
					tmp = new Cluster((State) ((Cluster) clusters.get(i)).state_out.get(j), machine, policy, failure, isPsuedo);
					//if(isPsuedo) tmp.Nfk = true;
					tmp.Nfk = true;
					tmp.id = auto_id;
					clusters.add(tmp);
				}
			}
		}
		if(isPsuedo){ //to have clusters for both Nfk and fk already existing
			int cluster_size = clusters.size();
			for(int i = 0; i < cluster_size; i++){
				try{
					//I believe this is right 
					tmp = (Cluster) ((Cluster) clusters.get(i)).clone();
				} catch(CloneNotSupportedException e){}

				tmp.Nfk = false;
				clusters.add(tmp);
			}
			
		}
		
		
	}
	
	private boolean alreadyCluster(State s){
		
		for(int i = 0; i < clusters.size(); i++){
			if(((Cluster) clusters.get(i)).startState.equals(s)) return true;
		}
		return false;
	}
	
	public boolean equals(Object o){
		
		if(o==null)return false;
		ClusterAutomata c_tmp;
		if (o instanceof ClusterAutomata) {
			c_tmp = (ClusterAutomata) o;
		}
		else return false;	
		if(!this.clusters.equals(c_tmp.clusters)) return false;
		if(!this.trans.equals(c_tmp.trans)) return false;
		if(!this.curCluster.equals(c_tmp.curCluster)) return false;
		
		return true;
	
	}

	public int hashCode(){
		int hash = 0;
		hash = trans.hashCode()+ clusters.hashCode();
		return hash;
	}

}
