package umdes;

import java.util.*;

/**
 * 
 * This class provides a representation of states that cannot be confusable according to
 * some specification.  The specification states are expanded in expand_spec 
 * 
 * @author John O'Brien May 20, 2009
 * 
 *
 */
public class T_spec {

	//two representations of the spec
	private HashMap<State, Set<State>> spec_map;
	private Vector<StatePair> spec_vec;
	private boolean empty;
	
	//private T_spec internal;
	private boolean t_spec_extended = false;
	
	public T_spec(){
		spec_map = new HashMap<State, Set<State>>();
		spec_vec = new Vector<StatePair>();
		empty = true;
	}
	
	public void clearSpec(){
		spec_map = new HashMap<State, Set<State>>();
		spec_vec = new Vector<StatePair>();
		empty = true;
	}
	
	public void addToSpec(State s1, State s2){	
		
		empty = false;
		if(spec_map.containsKey(s1)){ //update existing set
			Set<State> state_set = spec_map.get(s1);
			if(state_set.contains(s2)) return;
			state_set.add(s2);
			spec_map.put(s1, state_set);
		}
		else{ //make new set
			Set<State> state_set = new HashSet<State>();
			state_set.add(s2);
			spec_map.put(s1, state_set);
		}
		StatePair state = new StatePair(s1,s2);			
		spec_vec.add(state);
		
		//add the inverse also
		if(!spec_map.containsKey(s2)) this.addToSpec(s2,s1);
		if(!spec_map.get(s2).contains(s1)) this.addToSpec(s2, s1);
		t_spec_extended = false;
		
	}
	
	
	public Set<State> violate_set(State s){
		if(spec_map.containsKey(s))	return spec_map.get(s);
		else return null;
	}
	public Vector<StatePair> violate_vector(){
		return spec_vec;
	}
	
	public void extend_spec(UmdesAutomaton fsa){
			
		if(t_spec_extended) return; //already extended
		
		Vector<Transition> trans_s1 = new Vector<Transition>();
		Vector<Transition> trans_s2 = new Vector<Transition>();
		
	
		
		// the inverted pair will often will be next to the original, in this case we want to skip 
		State tmp = new State("");
		StatePair pair_temp = new StatePair(tmp,tmp); 		
		int current = 0;
		
		
		while(current < spec_vec.size()){

			State S1 = ((StatePair) spec_vec.get(current)).first;
			State S2 = ((StatePair) spec_vec.get(current)).second;
			
			if(pair_temp.first.equals(S2)&&pair_temp.second.equals(S1)) {current++; continue;}
			pair_temp = new StatePair(S1,S2);
			
			trans_s1 = fsa.getEvents().getAllTransitionsTo(S1);
			trans_s2 = fsa.getEvents().getAllTransitionsTo(S2);

			for(int i = 0; i<trans_s1.size();i++){
				for(int j = 0; j<trans_s2.size();j++){
					
					
					//there is an identical observable event
					 if(trans_s1.get(i).label.equals(trans_s2.get(j).label)){						 
						 	StatePair state = new StatePair(trans_s1.get(i).from,trans_s2.get(j).from);
						 	if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
						 		this.addToSpec(state.first, state.second);						 		
						 	}
					 }
					 //single unobservable from state 1
					 if((fsa.isUnobserv(trans_s1.get(i).label.getEvent()))){						 
						 	StatePair state = new StatePair(trans_s1.get(i).from,S2);
						 	if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
						 		this.addToSpec(state.first, state.second);						 		
						 	}
					 }
					 //single unobservable from state 2
					 if((fsa.isUnobserv(trans_s2.get(j).label.getEvent()))){						 
						 	StatePair state = new StatePair(S1,trans_s2.get(j).from);
						 	if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
						 		this.addToSpec(state.first, state.second);						 		
						 	}
					 }
					 //both unobservable events
					 if	(fsa.isUnobserv(trans_s1.get(i).label.getEvent())&&
						     fsa.isUnobserv(trans_s2.get(j).label.getEvent())){					 
						 	StatePair state = new StatePair(trans_s1.get(i).from,trans_s2.get(j).from);
						 	if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
						 		this.addToSpec(state.first, state.second);						 		
						 	}
					 }

					
				}
			}
			if(trans_s2==null||trans_s2.size()==0){
				for(int i = 0; i<trans_s1.size();i++){
					//second state is initial with no transition leading back to it
					if(fsa.isUnobserv(((Transition) trans_s1.get(i)).label.getEvent())){
						StatePair state = new StatePair(trans_s1.get(i).from,S2);
						if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
							this.addToSpec(state.first, state.second);						 		
						}
						continue;
					}
				}
			}
			if(trans_s1==null||trans_s1.size()==0){
				for(int j = 0; j<trans_s2.size();j++){
					//first state is initial with no transition leading back to it					 
					if((fsa.isUnobserv(trans_s2.get(j).label.getEvent()))){						 
						StatePair state = new StatePair(S1,trans_s2.get(j).from);
						if(!spec_map.containsKey(state.first)||!spec_map.get(state.first).contains(state.second)){
							this.addToSpec(state.first, state.second);						 		
						}
						continue;
					}
				}
			}
			
			current++;
		}		
		t_spec_extended = true;
		
	}
	
	public String toString() {
		String str  = "{";

		for(int i = 0; i < spec_vec.size(); i++){
			str += "(" + ((StatePair) spec_vec.get(i)).first.getName() + "," + 
				((StatePair) spec_vec.get(i)).first.getName() + ")";
			if(i!=spec_vec.size()-1) str+=",";
		}
		str += "}";
		
		return str;
	}



	public boolean isEmpty() {
		return empty;
	}


}
