/*** Giddes
 *     - Graphical Interface for Designing Discrete Event Systems
 *  S.Laurie Ricker Ph.D
 *  
 *  Developed by I. Fraser & W. Fowler 
 *           
 *  
 * reproduction without the express permission of Toba will
 * result in being taken outside and shot many times with a 
 * very tiny gun.
 ***/

//*********************************************************
// Documentation started February 9, 2002                 *
//*********************************************************

//*****************************************************************************************
///////////////////////////////////////////////////////////////////////********************
////////////////////////////////////////// Things still needed: Clock, and Clock Tick
///////////////////////////////////////////////////////////////////////********************
//*****************************************************************************************

package umdes;

/**
 * This Object describes the Timing Specification of a TransitionLabel
 * Used only for timed automata otherwise null
 *
 * @see UmdesAutomaton
 * @see RegExpr
 * @see Transition
 */
public class TimeSpec{
  //**************************************************************
  // private Members                                             *
  //**************************************************************
  
  public String debug; // pretend memeber
  
  /**
   * The upper bound of the timing constrant
   */
  private int upperBound;
  
  /**
   * The lower bound of the timing constrant
   */
  private int lowerBound;
  //**************************************************************
  // public members                                              *
  //**************************************************************
  
  /**
   * Creates a new TimeSpec Object with the given upper and lower bounds
   * At this time no error checking is done for negative values
   *
   * @param upper the upper bound for this timing specification
   * @param lower the lower bound for this timing specification
   */
  public TimeSpec(int upper, int lower){
    upperBound=upper;
    lowerBound=lower;
  } 
  
  /**
   * Sets the upper and lower bounds of this specification
   *
   * @param upper the upper bound for this timing specification
   * @param lower the lower bound for this timing specification
   */ 
  public void setBounds(int upper, int lower){
    upperBound=upper;
    lowerBound=lower;
  }
  
  /**
   * Returns the upper bound of this timing spec
   * 
   * @return The upper bound of this timing spec
   */
  public int getUpperBound(){
    return(upperBound);
  }
  
  /**
   * Returns the lower bound of this timing spec
   * 
   * @return The lower bound of this timing spec
   */
  public int getLowerBound(){
    return(lowerBound);
  }
  
  /**
   * Returns a string representaion of this timing spec
   */
  public String toString(){
    String temp= new String("THIS IS A DEBUG MESSAGE");
    return temp;
  }  
};
